--
-- fifo_pkg.vhd - package of FIFO components
--
-- SYNTHESIZABLE
--

library ieee;
use ieee.std_logic_1164.all;

package fifo is
  
    constant fifo_family_unknown  : integer := -1;
    constant fifo_family_virtex   : integer := 0;  -- Virtex/Virtex-E/Virtex-EM
    constant fifo_family_virtex2  : integer := 1;  -- Virtex-II
    constant fifo_family_virtex2p : integer := 2;  -- Virtex-IIPro
    constant fifo_family_virtex4  : integer := 3;  -- Virtex-4

    function afifo_depth(
        constant order : in natural)
    return natural;

    function sfifo_depth(
        constant order : in natural)
    return natural;

    component afifo
        generic(
            width    : in  natural;
            order    : in  natural;
            wpfl     : in  natural;
            wpfl2    : in  natural;
            rpel     : in  natural;
            family   : in  integer;
            safety   : in  boolean;
            fflush   : in  boolean);
        port(
            rst      : in  std_logic;
            wk       : in  std_logic;
            wd       : in  std_logic_vector(width - 1 downto 0);
            wadv     : in  std_logic;
            we       : out std_logic;
            wpf      : out std_logic;
            wpf2     : out std_logic;
            wf       : out std_logic;
            wovf     : out std_logic;
            wlevel   : out std_logic_vector(15 downto 0);
            rk       : in  std_logic;
            rq       : out std_logic_vector(width - 1 downto 0);
            radv     : in  std_logic;
            rflush   : in  std_logic;
            re       : out std_logic;
            rpe      : out std_logic;
            rf       : out std_logic;
            rudf     : out std_logic;
            rlevel   : out std_logic_vector(15 downto 0));
    end component;

    component sfifo
        generic(
            width    : in  natural;
            order    : in  natural;
            wpfl     : in  natural;
            rpel     : in  natural;
            family   : in  integer;
            safety   : in  boolean);
        port(
            rst      : in  std_logic;
            k        : in  std_logic;
            wd       : in  std_logic_vector(width - 1 downto 0);
            wadv     : in  std_logic;
            we       : out std_logic;
            wpf      : out std_logic;
            wf       : out std_logic;
            wovf     : out std_logic;
            rq       : out std_logic_vector(width - 1 downto 0);
            radv     : in  std_logic;
            rflush   : in  std_logic;
            re       : out std_logic;
            rpe      : out std_logic;
            rf       : out std_logic;
            rudf     : out std_logic;
            rlevel   : out std_logic_vector(15 downto 0));
    end component;

end;

package body fifo is

    function afifo_depth(
        constant order : in natural)
    return natural is
    begin
        return 2**order - 1;
    end;

    function sfifo_depth(
        constant order : in natural)
    return natural is
    begin
        return 2**order;
    end;

end;
